package com.persagy.fm.person.jms.handler;

import com.google.gson.Gson;
import com.persagy.fm.common.jms.MsgBaseHandler;
import com.persagy.fm.department.constant.enums.DepTypeEnum;
import com.persagy.fm.department.model.Department;
import com.persagy.fm.department.model.dto.QueryDepartmentDTO;
import com.persagy.fm.department.service.IDepartmentService;
import com.persagy.fm.person.constant.PersonJmsType;
import com.persagy.fm.person.jms.model.AccountCreateMsg;
import com.persagy.fm.person.jms.model.AccountUpdateMsg;
import com.persagy.fm.person.model.ConvertPersonTool;
import com.persagy.fm.person.model.Person;
import com.persagy.fm.person.model.dto.QueryPersonDTO;
import com.persagy.fm.person.service.IPersonQueryService;
import com.persagy.fm.person.service.IPersonUpsertService;
import com.persagy.fm.workresume.model.dto.AddWorkResumeDTO;
import com.persagy.fm.workresume.service.IWorkResumeService;
import com.rabbitmq.client.Channel;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * 账号消息处理类
 *
 * @author lixing
 * @version V1.0 2021/3/18 11:38 上午
 **/
@Service
@Slf4j
public class AccountMsgHandler {
    @Autowired
    private IPersonUpsertService personUpsertService;
    @Autowired
    private IPersonQueryService personQueryService;
    @Autowired
    private IWorkResumeService workResumeService;
    @Autowired
    private IDepartmentService departmentService;
    @Autowired
    MsgBaseHandler msgBaseHandler;

    /**
     * 处理账号创建消息(包含幂等性处理)
     *
     * @param accountCreateMsg 账号创建消息
     * @author lixing
     * @version V1.0 2021/3/18 11:40 上午
     */
    public void createAccount(AccountCreateMsg accountCreateMsg) {
        String accountId = accountCreateMsg.getId();
        /* 先查询后创建， 通过加锁的方式避免并行导致的重复创建 */
        synchronized (accountId.intern()) {
            // 查询集团下有没有使用消息中的账号id的用户
            QueryPersonDTO queryPersonDTO = new QueryPersonDTO();
            Person tmpPerson = personQueryService.queryByAccountId(accountId);

            if (tmpPerson != null) {
                log.error("已有人员使用该账号，拒绝创建人员");
                return;
            }

            // 找到集团下类型为集团的部门
            QueryDepartmentDTO queryDepartmentDTO = new QueryDepartmentDTO();
            queryDepartmentDTO.setType(DepTypeEnum.GROUP.getType());
            List<Department> departments = departmentService.queryDepartmentList(queryDepartmentDTO);
            if (CollectionUtils.isEmpty(departments)) {
                log.error("创建集团管理员发生异常，尚未创建集团");
            }
            Department group = departments.get(0);

            Person person = ConvertPersonTool.INSTANCE.convert2Entity(accountCreateMsg);
            personUpsertService.createPerson(person);

            // 创建人员的工作信息
            AddWorkResumeDTO addWorkResumeDTO = new AddWorkResumeDTO();
            addWorkResumeDTO.setHiredDate(new Date());
            addWorkResumeDTO.setDepId(group.getId());
            addWorkResumeDTO.setPersonId(person.getId());
            workResumeService.createWorkResume(addWorkResumeDTO);
        }
    }


    /**
     * 处理账号更新消息
     *
     * @param accountUpdateMsg 账号更新消息
     * @author lixing
     * @version V1.0 2021/3/18 11:40 上午
     */
    public void updateAccount(AccountUpdateMsg accountUpdateMsg) {
        // 根据账号id找到对应的人员信息
        Person person = personQueryService.queryByAccountId(accountUpdateMsg.getId());
        // 更新人员的账号信息
        if (person != null) {
            // 避免因为先查后更新与其他事务发生冲突
            String id = person.getId();
            person = new Person();
            person.setId(id);
            person.setUsername(accountUpdateMsg.getUsername());
            personUpsertService.updatePerson(person);
        }
    }

    /**
     * 监听saas队列
     *
     * @param message 消息体字符串
     * @param channel 通道对象
     * @param msg     消息对象
     * @author lixing
     * @version V1.0 2021/3/17 9:47 上午
     */
    @RabbitListener(queues = "${person.amqp.queue.saas}")
    public void process(String message, Channel channel, Message msg) {
        log.info("接收到运维平台账号消息：{}", message);
        try {
            // 切换数据源
            msgBaseHandler.resetDataSourceByMsg(message);
            // 获取消息类型
            String msgType = msgBaseHandler.getMsgType(message);
            Gson gson = new Gson();
            switch (msgType) {
                case PersonJmsType.ACCOUNT_CREATE:
                    AccountCreateMsg accountCreateMsg = gson.fromJson(message, AccountCreateMsg.class);
                    createAccount(accountCreateMsg);
                    break;
                case PersonJmsType.ACCOUNT_UPDATE:
                    AccountUpdateMsg accountUpdateMsg = gson.fromJson(message, AccountUpdateMsg.class);
                    updateAccount(accountUpdateMsg);
                    break;
                default:
                    break;
            }
            // 手动确认消息已消费
            channel.basicAck(msg.getMessageProperties().getDeliveryTag(), false);
        } catch (Exception e) {
            try {
                log.error("处理运维平台账号消息异常", e);
                // 将消费失败的消息移除消息队列，避免重复消费
                //                channel.basicReject(msg.getMessageProperties().getDeliveryTag(), false);
            } catch (Exception ex) {
                //                throw new BusinessException(ResponseCode.C0001.getCode(),
                //                        "账号消息从队列中移除失败，" + ex.getMessage());
            }
        }
    }
}
