package com.persagy.fm.saas.accountrole.service.impl;

import com.persagy.fm.common.constant.PageQueryConstants;
import com.persagy.fm.common.context.DefaultAppContext;
import com.persagy.fm.common.model.vo.SimpleObjVO;
import com.persagy.fm.common.response.FmResponseContent;
import com.persagy.fm.saas.account.constant.enums.AccountBelongEnum;
import com.persagy.fm.saas.accountrole.client.SaasAccountRoleClient;
import com.persagy.fm.saas.accountrole.constant.enums.AccountRoleTypeEnum;
import com.persagy.fm.saas.accountrole.model.SaasAccountRole;
import com.persagy.fm.saas.accountrole.model.dto.AddSaasAccountRoleDTO;
import com.persagy.fm.saas.accountrole.model.dto.DeleteSaasAccountRoleDTO;
import com.persagy.fm.saas.accountrole.model.dto.QuerySaasAccountRoleDTO;
import com.persagy.fm.saas.accountrole.model.dto.ResetSaasAccountRoleDTO;
import com.persagy.fm.saas.accountrole.model.vo.AccountRoleVO;
import com.persagy.fm.saas.accountrole.service.ISaasAccountRoleService;
import com.persagy.fm.saas.role.model.SaasRole;
import com.persagy.fm.saas.role.service.ISaasRoleService;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 账号-角色关联信息(SaasAccountRole) service层
 *
 * @author lixing
 * @version V1.0 2021-03-25 09:39:39
 */
@Service
public class SaasAccountRoleServiceImpl implements ISaasAccountRoleService {

    @Autowired
    SaasAccountRoleClient saasAccountRoleClient;
    @Autowired
    ISaasRoleService saasRoleService;

    /**
     * 创建账号-角色关联信息
     *
     * @return Long 账号-角色关联信息主键
     * @author lixing
     * @version V1.0 2021-03-25 09:39:39
     */
    @Override
    public String createSaasAccountRole(AddSaasAccountRoleDTO addSaasAccountRoleDTO) {
        addSaasAccountRoleDTO.setDefaultValueNoAccountId();
        FmResponseContent<String> saasAccountRole = saasAccountRoleClient.createSaasAccountRole(addSaasAccountRoleDTO);
        return saasAccountRole.getContent();
    }

    /**
     * 删除账号-角色关联信息
     *
     * @param accountId 账号主键
     * @author lixing
     * @version V1.0 2021-03-25 09:39:39
     */
    @Override
    public void deleteSaasAccountRole(String accountId) {
        DeleteSaasAccountRoleDTO deleteSaasAccountRoleDTO = new DeleteSaasAccountRoleDTO();
        deleteSaasAccountRoleDTO.setAccountId(accountId);
        deleteSaasAccountRoleDTO.setAccountBelong(AccountBelongEnum.BUSINESS.getType());
        deleteSaasAccountRoleDTO.setAppId(DefaultAppContext.getContext().getAppId());
        deleteSaasAccountRoleDTO.setGroupCode(DefaultAppContext.getContext().getGroupCode());
        saasAccountRoleClient.deleteSaasAccountRole(deleteSaasAccountRoleDTO);
    }

    /**
     * 查询账号-角色关联信息
     *
     * @return List<SaasAccountRole>
     * @author lixing
     * @version V1.0 2021-03-25 09:39:39
     */
    @Override
    public List<SaasAccountRole> querySaasAccountRoleList(QuerySaasAccountRoleDTO querySaasAccountRoleDTO) {
        querySaasAccountRoleDTO.setDefaultValueNoAccountId();
        FmResponseContent<List<SaasAccountRole>> responseContent = saasAccountRoleClient.querySaasAccountRoleList(querySaasAccountRoleDTO);
        return responseContent.getContent();
    }

    @Override
    public List<SaasAccountRole> querySaasAccountRoleListByAccountIdList(List<String> accountIdList) {
        if (CollectionUtils.isEmpty(accountIdList)) {
            return null;
        }
        QuerySaasAccountRoleDTO querySaasAccountRoleDTO = new QuerySaasAccountRoleDTO();
        querySaasAccountRoleDTO.setAccountIds(accountIdList);
        return querySaasAccountRoleList(querySaasAccountRoleDTO);
    }

    @Override
    public Map<String, AccountRoleVO> queryAccountRoleVoMapByAccountIdList(List<String> accountIdList) {
        if (CollectionUtils.isEmpty(accountIdList)) {
            return null;
        }
        List<SaasAccountRole> saasAccountRoles = querySaasAccountRoleListByAccountIdList(accountIdList);

        if (CollectionUtils.isEmpty(saasAccountRoles)) {
            return null;
        }

        // 查询这些角色的详情
        Set<String> roleIdSet = saasAccountRoles.stream().map(SaasAccountRole::getRoleId).collect(Collectors.toSet());
        Map<String, SaasRole> saasRoleMap = saasRoleService.querySaasRoleMapByRoleIdList(Lists.newArrayList(roleIdSet));

        // 将列表转换为map
        Map<String, AccountRoleVO> result = new HashMap<>(accountIdList.size());
        for (SaasAccountRole saasAccountRole : saasAccountRoles) {
            String accountId = saasAccountRole.getAccountId();
            AccountRoleVO accountRoleVO;
            if (result.containsKey(accountId)) {
                accountRoleVO = result.get(accountId);
            } else {
                accountRoleVO = new AccountRoleVO();
            }
            String roleId = saasAccountRole.getRoleId();
            String casType = saasAccountRole.getCasType();
            // 设置账号主岗信息
            if (AccountRoleTypeEnum.MAIN_DUTY.getType().equals(casType)) {
                accountRoleVO.setMainDuty(getRoleInfo(saasRoleMap, roleId));
            }
            // 设置账号副岗信息
            if (AccountRoleTypeEnum.OTHER_DUTY.getType().equals(casType)) {
                List<SimpleObjVO> otherDuties = accountRoleVO.getOtherDuties();
                if (otherDuties == null) {
                    otherDuties = Lists.newArrayList();
                }
                otherDuties.add(getRoleInfo(saasRoleMap, roleId));
                accountRoleVO.setOtherDuties(otherDuties);
            }
            // 设置账号其他角色信息
            if (AccountRoleTypeEnum.SYS_ROLE.getType().equals(casType)) {
                List<SimpleObjVO> otherRoles = accountRoleVO.getOtherRoles();
                if (otherRoles == null) {
                    otherRoles = Lists.newArrayList();
                }
                otherRoles.add(getRoleInfo(saasRoleMap, roleId));
                accountRoleVO.setOtherRoles(otherRoles);
            }
            result.put(accountId, accountRoleVO);
        }
        return result;
    }

    /**
     * 获取角色对象信息
     *
     * @param saasRoleMap 角色id和角色映射
     * @param roleId      角色id
     * @return SimpleObjVO 角色对象信息
     * @author lixing
     * @version V1.0 2021/4/1 10:34 上午
     */
    private SimpleObjVO getRoleInfo(Map<String, SaasRole> saasRoleMap, String roleId) {
        SimpleObjVO simpleObjVO = new SimpleObjVO();
        simpleObjVO.setId(roleId);
        SaasRole saasRole = saasRoleMap.get(roleId);
        String roleName = saasRole == null ? "" : saasRole.getRoleName();
        simpleObjVO.setName(roleName);
        return simpleObjVO;
    }

    @Override
    public void resetSaasAccountRole(String accountId, List<AddSaasAccountRoleDTO> roles) {
        resetSaasAccountRole(accountId, null, roles);
    }

    @Override
    public void resetSaasAccountRole(String accountId, String accountRoleType, List<AddSaasAccountRoleDTO> roles) {
        // 如果账号关联的所有角色都置空，调用运维平台的删除账号角色关联接口。运维平台重置账号关联角色的接口，不允许关联的角色为空
        if (CollectionUtils.isEmpty(roles)) {
            QuerySaasAccountRoleDTO querySaasAccountRoleDTO = new QuerySaasAccountRoleDTO();
            querySaasAccountRoleDTO.setDefaultValueNoAccountId();
            querySaasAccountRoleDTO.setCasType(accountRoleType);
            querySaasAccountRoleDTO.setAccountId(accountId);
            FmResponseContent<List<SaasAccountRole>> responseContent = saasAccountRoleClient.querySaasAccountRoleList(querySaasAccountRoleDTO);
            if (responseContent.getCount() > 0) {
                DeleteSaasAccountRoleDTO deleteSaasAccountRoleDTO = new DeleteSaasAccountRoleDTO();
                deleteSaasAccountRoleDTO.setDefaultValueNoAccountId();
                deleteSaasAccountRoleDTO.setAccountId(accountId);
                deleteSaasAccountRoleDTO.setCasType(accountRoleType);
                saasAccountRoleClient.deleteSaasAccountRole(deleteSaasAccountRoleDTO);
            }
        } else {
            ResetSaasAccountRoleDTO resetSaasAccountRoleDTO = new ResetSaasAccountRoleDTO();
            resetSaasAccountRoleDTO.setDefaultValueNoAccountId();
            resetSaasAccountRoleDTO.setAccountId(accountId);
            resetSaasAccountRoleDTO.setCasType(accountRoleType);
            resetSaasAccountRoleDTO.setRoles(roles);
            saasAccountRoleClient.resetSaasAccountRole(resetSaasAccountRoleDTO);
        }
    }

    @Override
    public List<String> queryAccountIdByRoleIdList(List<String> roleIds) {
        if (CollectionUtils.isEmpty(roleIds)) {
            return Lists.newArrayList();
        }

        if (roleIds.size() > PageQueryConstants.PARAM_MAX_SIZE) {
            throw new IllegalArgumentException("一次最多查询" + PageQueryConstants.PARAM_MAX_SIZE + "个角色关联的账号");
        }

        // 查询角色账号的关联信息
        QuerySaasAccountRoleDTO querySaasAccountRoleDTO = new QuerySaasAccountRoleDTO();
        querySaasAccountRoleDTO.setDefaultValueNoAccountId();
        querySaasAccountRoleDTO.setRoleIds(roleIds);
        FmResponseContent<List<SaasAccountRole>> responseContent = saasAccountRoleClient.
                querySaasAccountRoleList(querySaasAccountRoleDTO);
        List<SaasAccountRole> content = responseContent.getContent();

        if (CollectionUtils.isEmpty(content)) {
            return Lists.newArrayList();
        }

        return content.stream().map(SaasAccountRole::getAccountId).collect(Collectors.toList());
    }
}
